package linearalgebra;

import java.util.HashMap;
import java.util.Iterator;

public class LinearExpression {
	
	private HashMap<UnknownValue, LinearLiteral> m_literals = new HashMap<UnknownValue, LinearLiteral>();
	private double m_value = 0.0;
	
	public double getConstant() {
		return m_value;
	}
	
	public int getVariableCount() {
		return m_literals.size();
	}
	
	public void resolveBoundUnknowns() {
		Iterator iter = m_literals.values().iterator();
		while( iter.hasNext() ) {
			LinearLiteral l = (LinearLiteral)iter.next();
			if( l.variable.isBound() ) {
				m_value += l.coefficient * l.variable.getNumericValue();
				iter.remove();
			}
		}
	}
	
	public double getCoefficient( UnknownValue variable ) {
		LinearLiteral l = m_literals.get(variable);
		if( l == null ) {
			return 0.0;
		}
		return l.coefficient;
	}
	
	public void remove( UnknownValue variable ) {
		m_literals.remove(variable);
	}
	
	public void substitute( UnknownValue variable, LinearExpression exp ) {
		assert( !exp.m_literals.containsKey(variable) );
		double c = getCoefficient(variable);
		if( !Util.isZero(c) ) {
			remove( variable );
			Iterator iter = exp.m_literals.values().iterator();
			while( iter.hasNext() ) {
				LinearLiteral l = (LinearLiteral)iter.next();
				add( c*l.coefficient, l.variable );
			}
			add( c*exp.m_value );
		}
	}
	
	public void add( LinearExpression exp, double factor ) {
		Iterator iter = exp.m_literals.values().iterator();
		while( iter.hasNext() ) {
			LinearLiteral l = (LinearLiteral)iter.next();
			add( factor*l.coefficient, l.variable );
		}
		add( exp.m_value );
	}
	
	public void add( LinearExpression exp ) {
		add( exp, 1.0 );
	}
	
	public void add( double coefficient, UnknownValue variable ) {
		if( !Util.isZero( coefficient ) ) {
			LinearLiteral l = m_literals.get(variable);
			if( l == null ) {
				m_literals.put( variable, new LinearLiteral( coefficient, variable ) );
			} else {
				l.coefficient += coefficient;
				if( Util.isZero(l.coefficient) ) {
					m_literals.remove(variable);
				}
			}
		}
	}
	
	public void add( double value ) {
		m_value += value;
	}
	
	public void scaleAll( double d ) {
		Iterator iter = m_literals.values().iterator();
		while( iter.hasNext() ) {
			LinearLiteral l = (LinearLiteral)iter.next();
			l.coefficient *= d;
		}
		m_value *= d;
	}
	
	public String toString() {
		StringBuffer rc = new StringBuffer();
		Iterator iter = m_literals.values().iterator();
		if( iter.hasNext() ) {
			LinearLiteral l = (LinearLiteral)iter.next();
			double v = l.coefficient;
			if( v < 0 ) {
				rc.append("-");
				v = Math.abs(v);
			}
			printLiteral( v, l.variable, rc );
		} else {
			rc.append(m_value);
			return rc.toString();
		}
		while( iter.hasNext() ) {
			LinearLiteral l = (LinearLiteral)iter.next();
			double v = l.coefficient;
			if( v > 0 ) {
				rc.append(" + ");
			} else {
				rc.append(" - ");
			}		
			v = Math.abs(v);
			printLiteral( v, l.variable, rc );
		}
		if( !Util.isZero(m_value) ) {
			double v = m_value;
			if( v > 0 ) {
				rc.append(" + ");
			} else {
				rc.append(" - ");
			}		
			v = Math.abs(v);
			rc.append(m_value);
		}
		return rc.toString();
	}
	
	private static void printLiteral( double c, UnknownValue v, StringBuffer s ) {
		if( !Util.isZero(c-1.0) ) {
			s.append(c);
			s.append("*");
		}
		s.append(v.getName());		
	}
	
	private static class LinearLiteral {
		public double coefficient;
		public UnknownValue variable;
		public LinearLiteral( double coefficient, UnknownValue variable ) {
			this.coefficient = coefficient;
			this.variable = variable;
		}
	}
	
	public void substituteConstraint( UnknownValue v ) {
		if( m_literals.containsKey( v ) ) {
			throw new Error();
		}
		LinearExpression temp = new LinearExpression();
		Iterator iter = m_literals.values().iterator();
		while( iter.hasNext() ) {
			LinearLiteral l = (LinearLiteral)iter.next();
			LinearExpression constraint = l.variable.getConstraint();
			if( constraint != null ) {
				// substitution would result in a zero on the left hand side
				// this case shouldn't happen for a properly eliminated matrix
				if( !Util.isEqual(constraint.getCoefficient(v)*l.coefficient, 1.0) ) {
					temp.add( constraint, l.coefficient );
					iter.remove();
				}
			}
        }
        add(temp);
	}
}