package linearalgebra.numeric;

import linearalgebra.*;
import linearalgebra.exception.*;

public	final class	NumericRow 
		implements	NumericRowInterface {
	
	private NumericVectorInterface m_vector;
	
	public NumericRow( NumericVectorInterface proxy ) {
		m_vector = proxy;
	}
	
	public NumericRow( double[] copy ) {
		m_vector = new NumericVector(copy);
	}
	
	public NumericRow( NumericMatrixInterface copy, int row ) {
		m_vector = new NumericVector( copy.getNumericRowView(row).getVectorView() );
	}
	
	public NumericRow( NumericRow copy ) {
		this( (NumericVectorInterface)(copy.m_vector.clone()) );
	}
	
	public Object clone() {
		try {
			Object rc = super.clone();
			((NumericRow)rc).m_vector = (NumericVectorInterface)m_vector.clone();
			return rc;
		} catch( CloneNotSupportedException e ) {
			throw new InternalError("CloneNotSupported unexpectedly caught in NumericRow.clone()");
		}
	}
	
	public VectorInterface getVectorView() {
		return getNumericVectorView();
	}
	
	public NumericVectorInterface getNumericVectorView() {
		return m_vector;
	}
	
	public int getRowCount() {
		return 1;
	}
	
	public double getColumnValue( int col ) {
		return m_vector.getValue(col);
	}
	
	public int getDimension() {
		return getRowCount();
	}
	
	public void multiplyRow( int index, double scalar ) {
		m_vector.scaleDimension(index, scalar);
	}
	
	public void combineRows( int srcRow, int dstRow, double scalar ) {
		m_vector.linearCombinationDimension(srcRow, dstRow, scalar);
	}
	
	public double getRowValue( int row ) {
		return m_vector.getValue(row);
	}

	public void swapRows( int row1, int row2 ) {
	}	

	public void swapRows( PermutationMatrix p ) {
	}	
	
	public boolean hasRowPivot( int row ){
		for( int i = 0; i < getColumnCount(); i++ ) {
			if( m_vector.getValue(i) != 0 ) {
				return true;
			}
		}
		return false;
	}
	
	public boolean hasColumnPivot( int col ){		
		if( m_vector.getValue(col) != 0 ) {
			return true;
		}
		return false;
	}
	
	public int getRowLeftPivotIndex( int row ){
		for( int i = 0; i < getColumnCount(); i++ ) {
			if( m_vector.getValue(i) != 0 ) {
				return i;
			}
		}
		throw new NoPivotError();
	}
	
	public int getRowRightPivotIndex( int row ){
		for( int i = 0; i < getColumnCount(); i++ ) {
			if( m_vector.getValue(i) != 0 ) {
				return i;
			}
		}
		throw new NoPivotError();
	}
	
	public int getColumnTopPivotIndex( int col ){
		if( hasColumnPivot(col) ) {
			return col;
		}
		throw new NoPivotError();
	}
	
	public int getColumnBottomPivotIndex( int col ){
		if( hasColumnPivot(col) ) {
			return col;
		}
		throw new NoPivotError();
	}
	
	public int getColumnCount() {
		return m_vector.getDimension();
	}
	
	// operations for eliminations
	public double getElement( int row, int col ) {
		if( col != 0 ) {
			throw new ArrayIndexOutOfBoundsException();
		}
		return getRowValue(row);
	}
		
	// arithmetic operations

	public NumericMatrixInterface plus( NumericMatrixInterface m ) {
		NumericRow rc = new NumericRow(this);
		return rc.plusEquals(m);
	}
	
	public NumericMatrixInterface plusEquals( NumericMatrixInterface m ) {
		if( getRowCount() != m.getRowCount() || 1 != m.getColumnCount() ) {
			throw new SizeMismatchError("Matrices must have similar dimensions for addition.");
		}
		NumericVectorInterface vector2 = m.getNumericRowView(0).getNumericVectorView();
		m_vector.linearCombinationEquals(vector2, 1.0);
		return this;
	}
	
	public NumericMatrixInterface minus( NumericMatrixInterface m ) {
		NumericRow rc = new NumericRow(this);
		return rc.minusEquals(m);
	}
	
	public NumericMatrixInterface minusEquals( NumericMatrixInterface m ) {
		if( getRowCount() != m.getRowCount() || 1 != m.getColumnCount() ) {
			throw new SizeMismatchError("Matrices must have similar dimensions for subtraction.");
		}
		NumericVectorInterface vector2 = m.getNumericRowView(0).getNumericVectorView();
		m_vector.linearCombinationEquals(vector2, -1.0);
		return this;
	}
	
	public NumericMatrixInterface multiply( double scalar ) {
		NumericRow rc = new NumericRow(this);
		return rc.multiplyEquals(scalar);
	}
	
	public NumericMatrixInterface multiplyEquals( double scalar ) {
		m_vector.scale(scalar);
		return this;
	}

	public NumericMatrixInterface product( NumericMatrixInterface right ) {
		if( getColumnCount() != right.getRowCount() ) {
			throw new SizeMismatchError("Row count of first matrix must equal column count of second matrix for cross product.");
		}
		Matrix rc = new Matrix( getRowCount(), right.getColumnCount() );
		
		for( int col = 0; col < rc.getColumnCount(); col++ ) {
			NumericVectorInterface colVector = right.getNumericColumnView(col).getNumericVectorView();
			rc.setElement(0, col, m_vector.innerProduct(colVector));
		}		
		return rc;
	}
	
	public NumericMatrixInterface power( int power ) {
		NumericMatrixInterface rc = this;
		for( int i = 1; i < power; i++ ) {
			rc = rc.product(this);
		}
		return rc;
	}

	public boolean equals( Object o ) {
		boolean rc = false;
		if( o instanceof NumericMatrixInterface ) {
			NumericMatrixInterface m = (NumericMatrixInterface)o;
			if( getRowCount() == m.getRowCount() && getColumnCount() == m.getColumnCount() ) {
				if( m_vector.equals( m.getNumericColumnView(0).getVectorView() ) ) {
					rc = true;
				}
			}
		}
		return rc;
	}
	
	public NumericColumnInterface getNumericColumnView( int col ) {
		SingleElementVector v = new SingleElementVector( m_vector, col );
		return new NumericColumn( v );
	}
	
	public ColumnInterface getColumnView( int col ) {
		return getNumericColumnView(col);
	}
	
	public NumericRowInterface getNumericRowView( int row ) {
		if( row != 0 ) { 
			throw new ArrayIndexOutOfBoundsException(); 
		}
		return this;
	}
	
	public RowInterface getRowView( int row ) {
		return getNumericRowView( row );
	}
	
	public String toString() {
		StringBuffer rc = new StringBuffer();
		for( int col = 0; col < getColumnCount(); col++ ) {
			rc.append(getColumnValue(col)+"\t");
		}
		rc.append("\n");
		return rc.toString();
	}
	
    public MatrixInterface factorL(PermutationMatrix p) {
        return factorL(p, true);
    }

	public MatrixInterface factorL(PermutationMatrix p, boolean partialPivot) {
		Matrix rc = new Matrix( new double[][] {{1.0}} );
		return rc;
	}

	// copied from NumericColumn.factorL()
	// modify for U and row
	public MatrixInterface factorU(PermutationMatrix p) {
		Matrix rc = new Matrix( getDimension() );	// creates an identity matrix
		
		int pivot_index = 0;
		while( getElement(pivot_index, 0) == 0.0 ) {
			++pivot_index;
			if( pivot_index >= getDimension() ) {
				return rc;
			}
		}
		double pivot = getElement( pivot_index, 0 );
		for( int col = pivot_index; col < rc.getRowCount(); col++ ) {
			rc.setElement(pivot_index, col, getElement(0, col)/pivot);
		}
		for( int row = 0; row < rc.getRowCount(); row++ ) {
			m_vector.setValue(row, 0.0);
		}
		m_vector.setValue(pivot_index, pivot);

		return rc;
	}

	public MatrixInterface factorInverse() throws linearalgebra.exception.UninvertibleMatrixException {
		throw new linearalgebra.exception.UninvertibleMatrixException();
	}
	
	public MatrixInterface getTranspose() {
		return new NumericColumn( (NumericVector)m_vector.clone() );
	}
	
	public boolean isIdentity() {
		return ( getDimension() == 1 ) && ( getColumnValue(0) == 1.0 );
	}
	
	public boolean isUpperRightTriangular() {
		boolean rc = true;
		for( int i = 1; i < getDimension(); i++ ) {
			if( getColumnValue(i) != 0.0 ) {
				rc = false;
				break;
			}
		}
		return rc;
	}
	
	public boolean isLowerLeftTriangular() {
		return true;
	}
	
	public boolean isSymmetric() {
		return false;
	}
	
	public boolean isPositiveDefinite() {
		return false;
	}
	
    public void incrementElement(int row, int col, double value) {
    	if( row == 0 ) {
    		m_vector.setValue( col, m_vector.getValue(col)+value );
    	} else {
    		throw new ArrayIndexOutOfBoundsException();
    	}
    }
    
    public void scaleElement(int row, int col, double scalar) {
    	if( row == 0 ) {
    		m_vector.scaleDimension(col, scalar);
    	} else {
    		throw new ArrayIndexOutOfBoundsException();
    	}
    }
    
    public void setElement(int row, int col, double element) {
    	if( row == 0 ) {
    		m_vector.setValue(col, element);
    	} else {
    		throw new ArrayIndexOutOfBoundsException();
    	}
    }
    
    public MatrixInterface getProjectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public MatrixInterface getOrthogonalProjectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public MatrixInterface getReflectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public boolean isIdempotent() {
    	throw new UnsupportedOperationError();
    }

	public boolean isOrthogonal() {
		return false;
	}
}